/*
 * Copyright 2012 Martin Skurla
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 */
package org.crazyjavahacking.astvisualizer.impl.javac;


import org.crazyjavahacking.astvisualizer.api.model.EdgeModel;
import org.crazyjavahacking.astvisualizer.api.model.NodeModel;
import org.jgrapht.DirectedGraph;
import org.jgrapht.graph.SimpleDirectedGraph;
import org.jgrapht.graph.UnmodifiableDirectedGraph;


/**
 * Represents customized AST.
 *
 * @author Martin
 */
final class CustomizedAST {

    //-----------------------------------------------------------------------------------------------------------------
    // Members.
    //-----------------------------------------------------------------------------------------------------------------
    private final DirectedGraph<NodeModel, EdgeModel> directedGraph =
            new SimpleDirectedGraph<NodeModel, EdgeModel>(EdgeModel.class);

    private NodeModel lastNode = null;


    //-----------------------------------------------------------------------------------------------------------------
    // Public Methods.
    //-----------------------------------------------------------------------------------------------------------------
    /**
     * Adds node to customized AST.
     *
     * @param node node
     */
    public void addNode(NodeModel node) {
        directedGraph.addVertex(node);

        lastNode = node;
    }

    /**
     * Adds edge to customized AST. Given nodes must be first added using <code>addNode(NodeModel)</code> method.
     *
     * @param startNode start node
     * @param endNode   end node
     * @param edge      edge
     */
    public void addEdge(NodeModel startNode, NodeModel endNode, EdgeModel edge) {
        directedGraph.addEdge(startNode, endNode, edge);
    }

    /**
     * Returns number of nodes.
     *
     * @return number of nodes
     */
    public int getNumberOfNodes() {
        return directedGraph.vertexSet().size();
    }

    /**
     * Returns last added node.
     *
     * @return last added node
     */
    public NodeModel getLastAddedNode() {
        return lastNode;
    }

    /**
     * Transforms customized AST into <b>unmodifiable</b> directed graph.
     *
     * @return unmodifiable directed graph
     */
    public DirectedGraph<NodeModel, EdgeModel> toDirectedGraph() {
        return new UnmodifiableDirectedGraph<NodeModel, EdgeModel>(directedGraph);
    }
}
