/*
 * Copyright 2012 Martin Skurla
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 */
package org.crazyjavahacking.astvisualizer.impl.javac.model;


import java.util.Collections;
import java.util.List;

import javax.lang.model.element.Modifier;
import javax.lang.model.element.Name;
import javax.lang.model.type.TypeKind;

import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.Getter;


/**
 * Edge data factory.
 *
 * @author Martin
 */
public final class EdgeDataFactory {

    //-----------------------------------------------------------------------------------------------------------------
    // Constructors.
    //-----------------------------------------------------------------------------------------------------------------
    private EdgeDataFactory() {}


    //-----------------------------------------------------------------------------------------------------------------
    // Public Methods.
    //-----------------------------------------------------------------------------------------------------------------
    /**
     * Creates IEdgeData from given transition name.
     *
     * @param transitionName transition name
     *
     * @return IEdgeData
     */
    public static IEdgeData fromTree(String transitionName) {
        return new TreeEdgeData(transitionName);
    }

    /**
     * Creates List of IEdgeData objects from given transition name and number of times.
     *
     * @param transitionName transition name
     * @param times          times
     *
     * @return list of IEdgeData objects
     */
    public static List<IEdgeData> fromTreeMultipleTimes(String transitionName, int times) {
        return Collections.nCopies(times, EdgeDataFactory.fromTree(transitionName));
    }

    /**
     * Creates IEdgeData from given transition name and javax.lang.model.element.Name.
     *
     * @param transitionName transition name
     * @param name           name
     *
     * @return IEdgeData
     */
    public static IEdgeData fromName(String transitionName, Name name) {
        return new NameEdgeData(transitionName,
                                name);
    }

    /**
     * Creates IEdgeData from given transition name and javax.lang.model.element.Modifier.
     *
     * @param transitionName transition name
     * @param modifier       modifier
     *
     * @return IEdgeData
     */
    public static IEdgeData fromModifier(String transitionName, Modifier modifier) {
        return new ModifierEdgeData(transitionName,
                                    modifier);
    }

    /**
     * Creates IEdgeData from given stansition name and javax.lang.model.type.TypeKind.
     *
     * @param transitionName transition name
     * @param typeKind       type kind
     *
     * @return IEdgeData
     */
    public static IEdgeData fromTypeKind(String transitionName, TypeKind typeKind) {
        return new TypeKindEdgeData(transitionName,
                                    typeKind);
    }


    //-----------------------------------------------------------------------------------------------------------------
    // Inner classes.
    //-----------------------------------------------------------------------------------------------------------------
    @AllArgsConstructor(access= AccessLevel.PRIVATE)
    @Getter
    private static final class TreeEdgeData implements IComSunSourceTreeTreeEdgeData {

        private final String transitionName;
    }

    @AllArgsConstructor(access= AccessLevel.PRIVATE)
    @Getter
    private static final class NameEdgeData implements IJavaxLangModelElementNameEdgeData {

        private final String transitionName;
        private final Name name;
    }

    @AllArgsConstructor(access= AccessLevel.PRIVATE)
    @Getter
    private static final class ModifierEdgeData implements IJavaxLangModelElementModifierEdgeData {

        private final String transitionName;
        private final Modifier modifier;
    }

    @AllArgsConstructor(access= AccessLevel.PRIVATE)
    @Getter
    private static final class TypeKindEdgeData implements IJavaxLangModelTypeTypeKindEdgeData {

        private final String transitionName;
        private final TypeKind typeKind;
    }
}
